-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8

local RC = RaidChecklist

-- Map from GUID to name
RC.provider_names = {}

-- Map from (de)buff key to list of GUIDs currently providing said de(buff)
RC.providers = {}
for i,b in ipairs (RC.buffs) do RC.providers[b] = {} end
for i,b in ipairs (RC.debuffs) do RC.providers[b] = {} end

local function remove_provider (rguid)
  -- Iterate over all the (de)buffs
  for key,guidlist in pairs (RC.providers) do
    guidlist[rguid] = nil
  end
  if (RC.OnRefresh) then RC:OnRefresh () end
end

local function add_provider (buff, guid)
  RC.providers[buff][guid] = true
  if (RC.OnRefresh) then RC:OnRefresh () end
end

-- Works out which spec is used based on talent point allocations.
-- Returns 0 if no main tree is spec'd into, else 1-3 indicating tree.
local function which_spec (n1, n2, n3)
  return (n1 > n2 and n1 > n3 and 1) or (n2 > n1 and n2 > n3 and 2) or (n3 > n1 and n3 > n2 and 3) or 0
end

-- Matrix based on select(2, UnitBaseClass(unit)) and spec (0-4) to a list of (de)buffs provided
-- This is almost a duplication of the "availablefrom" table over in raid_checklist, but that one
-- has to be human readable and there was just no neat way to satisfy both in a reasonable manner :(
local matrix = {
  WARRIOR = {
    [0] = { "AGISTR", "STAM", "ATTACKSPD" }, -- All warriors can provide these even without a spec
    [1] = { "AGISTR", "STAM", "ATTACKSPD", "PHYSDMGTAKEN", "BLEEDDMGTAKEN", "MORTALSTRIKE" }, -- Arms
    [2] = { "AGISTR", "STAM", "ATTACKSPD", "CRIT", "MORTALSTRIKE" }, -- Fury
    [3] = { "AGiSTR", "STAM", "ATTACKSPD", "DEMOROAR", "SUNDER" }, -- Protection
  },
  HUNTER = {
    [0] = { "MORTALSTRIKE" },
    [1] = { "MORTALSTRIKE", "ALLDMG" }, -- BM
    [2] = { "MORTALSTRIKE", "ATTACKPWR" }, -- Marksman
    [3] = { "MORTALSTRIKE", "PHYSHASTE" }, -- Survival
  },
  DEATHKNIGHT = {
    [0] = { "AGISTR", "ATTACKPWR", "ATTACKSPD" },
    [1] = { "AGISTR", "ATTACKPWR", "ATTACKSPD", "DEMOROAR" }, -- Blood
    [2] = { "AGISTR", "ATTACKPWR", "ATTACKSPD", "PHYSHASTE", "PHYSDMGTAKEN" }, -- Frost
    [3] = { "AGISTR", "ATTACKPWR", "ATTACKSPD", "SPELLDMGTAKEN" }, -- Unholy
  },
  PRIEST = {
    [0] = { "STAM" },
    [1] = { "STAM" }, -- Discipline
    [2] = { "STAM" }, -- Holy
    [3] = { "STAM", "SPELLHASTE", "REPLENISHMENT", "MORTALSTRIKE" }, -- Shadow
  },
  SHAMAN = {
    [0] = { "AGISTR", "SPELLPWR6", "PHYSHASTE", "SPELLHASTE", "BLOODLUST", "MANAREGEN" },
    [1] = { "AGISTR", "SPELLPWR6", "PHYSHASTE", "SPELLHASTE", "BLOODLUST", "MANAREGEN", "CRIT", "SPELLPWR10" }, -- Elemental
    [2] = { "AGISTR", "SPELLPWR6", "PHYSHASTE", "SPELLHASTE", "BLOODLUST", "MANAREGEN", "ATTACKPWR" }, -- Enhancement
    [3] = { "AGISTR", "SPELLPWR6", "PHYSHASTE", "SPELLHASTE", "BLOODLUST", "MANAREGEN" }, -- Restoration
  },
  PALADIN = {
    [0] = { "STATS", "ATTACKPWR", "MANAREGEN" },
    [1] = { "STATS", "ATTACKPWR", "MANAREGEN" }, -- Holy
    [2] = { "STATS", "ATTACKPWR", "MANAREGEN", "ATTACKSPD", "DEMOROAR" }, -- Protection
    [3] = { "STATS", "ATTACKPWR", "MANAREGEN", "REPLENISHMENT", "ALLDMG" }, -- Retribution
  },
  DRUID = {
    [0] = { "STATS" },
    [1] = { "STATS", "SPELLHASTE", "SPELLDMGTAKEN" }, -- Balance
    [2] = { "STATS", "CRIT", "BLEEDDMGTAKEN", "ATTACKSPD", "DEMOROAR", "SUNDER" }, -- Feral (demo/sunder only in bear though)
    [3] = { "STATS", "REPLENISHMENT" }, -- Restoration
  },
  WARLOCK = {
    [0] = { "SPELLDMGTAKEN", "CASTSPD" },
    [1] = { "SPELLDMGTAKEN", "CASTSPD" }, -- Affliction
    [2] = { "SPELLDMGTAKEN", "CASTSPD", "SPELLPWR10"  }, -- Demonology
    [3] = { "SPELLDMGTAKEN", "CASTSPD", "REPLENISHMENT", "SPELLCRIT" }, -- Destruction (spell crit accessible from other trees though "Shadow and Flame")
  },
  ROGUE = {
    [0] = { "CASTSPD", "MORTALSTRIKE", "SUNDER" },
    [1] = { "CASTSPD", "MORTALSTRIKE", "SUNDER", "SPELLDMGTAKEN" }, -- Assasination
    [2] = { "CASTSPD", "MORTALSTRIKE", "SUNDER", "PHYSDMGTAKEN" }, -- Combat
    [3] = { "CASTSPD", "MORTALSTRIKE", "SUNDER", "CRIT", "BLEEDDMGTAKEN" }, -- Subtlety
  },
  MAGE = {
    [0] = { "SPELLPWR6", "BLOODLUST", "MANA" },
    [1] = { "SPELLPWR6", "BLOODLUST", "MANA", "CASTSPD", "ALLDMG" }, -- Arcane
    [2] = { "SPELLPWR6", "BLOODLUST", "MANA", "SPELLPWR10", "SPELLCRIT" }, -- Fire
    [3] = { "SPELLPWR6", "BLOODLUST", "MANA", "REPLENISHMENT" }, -- Frost
  }
}

local function provides (class, spec)
  return matrix[class][spec]
end

RC.OnTalentUpdate = function (self, event, guid, unit, spec, n1, n2, n3)
  -- Update the talent status
  RC.missing_talents_for = RC.gt:GetTalentMissingNames ()

  -- Clear any previous entries for this guid
  remove_provider (guid)

  local spec = which_spec (n1, n2, n3)
  local class = select (2, UnitClassBase (unit))
  local list = matrix[class][spec]
  for i,key in ipairs (list) do
    add_provider (key, guid)
  end
end

RC.OnTalentAdd = function (self, event, guid, unit, name, realm)
  local fullname = (realm and name.."-"..realm) or name
  RC.provider_names[guid] = ClassColorString (select (2,UnitClassBase (unit)))..fullname.."|r"

  -- Update based on class alone to start off with. Better than waiting until in inspect range!
  RC:OnTalentUpdate ("fake", guid, unit, 0, 0, 0, 0)

  --RaidChecklistButton:UNIT_PET(unit) -- We are probably better off waiting for a proper UNIT_PET event I think

  --DEFAULT_CHAT_FRAME:AddMessage ("RC: discovered provider GUID "..(guid or "nil")..", named "..fullname)
end

RC.OnTalentRemove = function (self, event, guid, name, realm)

  remove_provider (guid)
  RC.provider_names[guid] = nil

  -- Update the talent status
  RC.missing_talents_for = RC.gt:GetTalentMissingNames ()

  --DEFAULT_CHAT_FRAME:AddMessage ("RC: removed provider GUID "..(guid or "nil"))
end

RC.gt = LibStub ("LibGroupTalents-1.0")
RC.gt.RegisterCallback (RC, "LibGroupTalents_Update", "OnTalentUpdate")
RC.gt.RegisterCallback (RC, "LibGroupTalents_Add", "OnTalentAdd")
RC.gt.RegisterCallback (RC, "LibGroupTalents_Remove", "OnTalentRemove")

